#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glib.h>

#include <string.h>
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "css.h"

extern GtkWidget *window;

GdkPixmap *preview_pixmap;

gboolean draw_border = 1;
gint border_width = 2, padding = 6;


void update_kontrast (void) {
	gchar str[128];
	gfloat val;
	
	val = css_get_brightness_dif(main_css, CSS_CTEXT, CSS_CBG);
	sprintf(str, "%s%.2f%s", (val < 125 && val > -125)?"<b>":"", val,
			(val < 125 && val > -125)?"</b>":"");
	gtk_label_set_markup(GTK_LABEL(lookup_widget(window, "label_jas1")), str);
	
	val = css_get_color_dif(main_css, CSS_CTEXT, CSS_CBG);
	sprintf(str, "%s%.0f%s", (val < 500 && val > -500)?"<b>":"", val,
			(val < 500 && val > -500)?"</b>":"");
	gtk_label_set_markup(GTK_LABEL(lookup_widget(window, "label_barva1")), str);

	
	val = css_get_brightness_dif(main_css, CSS_CBORDER, CSS_CBG);
	sprintf(str, "%s%.2f%s", (val < 125 && val > -125)?"<b>":"", val,
			(val < 125 && val > -125)?"</b>":"");
	gtk_label_set_markup(GTK_LABEL(lookup_widget(window, "label_jas2")), str);

	val = css_get_color_dif(main_css, CSS_CBORDER, CSS_CBG);
	sprintf(str, "%s%.0f%s", (val < 500 && val > -500)?"<b>":"", val,
			(val < 500 && val > -500)?"</b>":"");
	gtk_label_set_markup(GTK_LABEL(lookup_widget(window, "label_barva2")), str);


	val = css_get_brightness_dif(main_css, CSS_CBORDER, CSS_CAROUND);
	sprintf(str, "%s%.2f%s", (val < 125 && val > -125)?"<b>":"", val,
			(val < 125 && val > -125)?"</b>":"");
	gtk_label_set_markup(GTK_LABEL(lookup_widget(window, "label_jas3")), str);

	val = css_get_color_dif(main_css, CSS_CBORDER, CSS_CAROUND);
	sprintf(str, "%s%.0f%s", (val < 500 && val > -500)?"<b>":"", val,
			(val < 500 && val > -500)?"</b>":"");
	gtk_label_set_markup(GTK_LABEL(lookup_widget(window, "label_barva3")), str);

}

void update_css (void) {
	gchar str[2048];

	css_generate_str(main_css, str);
	gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(
			lookup_widget(window, "textview_css"))), str, -1);
}

void update_preview (void) {
	GtkWidget* drawing_area = lookup_widget(window, "drawingarea_nahled");
	GtkWidget* tmp = gtk_widget_ref(window);
	GdkGC *gc = gdk_gc_new(window->window);
	GdkRectangle  update_rect;
	GdkColor okoli_color, border_color, bg_color, text_color;
	gint tw = 120, th = 40, w, h;
	gchar text[1024];
	GdkFont *text_font;
	PangoLayout *layout;
	PangoFontDescription * font;
	PangoContext *context;

	/* recreate pixmap */
	if (preview_pixmap) {
		gdk_pixmap_unref (preview_pixmap);
	} 
	
	w = drawing_area->allocation.width;
	h = drawing_area->allocation.height;
	
	preview_pixmap = gdk_pixmap_new (drawing_area->window, w, h, -1);
	
	/* take color from color button */
	gtk_color_button_get_color(GTK_COLOR_BUTTON(lookup_widget(window, "colorbutton_okoli")),
			&okoli_color);
	gtk_color_button_get_color(GTK_COLOR_BUTTON(lookup_widget(window, "colorbutton_ramecek")),
			&border_color);
	gtk_color_button_get_color(GTK_COLOR_BUTTON(lookup_widget(window, "colorbutton_pozadi")),
			&bg_color);
	gtk_color_button_get_color(GTK_COLOR_BUTTON(lookup_widget(window, "colorbutton_text")),
			&text_color);

	/* get font */
	text_font = gdk_font_load("-*-helvetica-medium-r-*-*-17-*-*-*-*-*-*-*");
/*	text_font = gdk_fontset_load(gtk_font_button_get_font_name(GTK_FONT_BUTTON(
				lookup_widget(window, "fontbutton_text"))));

*/
/*	fprintf(stderr, "font: %s\n", gtk_font_button_get_font_name(GTK_FONT_BUTTON(
				lookup_widget(window, "fontbutton_text"))));
*/

	/* get text */
	strcpy(text, gtk_entry_get_text(GTK_ENTRY(lookup_widget(window, "entry_text"))));
	tw = gdk_string_width(text_font, text);	
	th = gdk_string_height(text_font, text);


	/* okoli */
	gtk_widget_modify_fg (tmp, GTK_STATE_NORMAL, &okoli_color);
	gdk_draw_rectangle(preview_pixmap,
			tmp->style->fg_gc[GTK_STATE_NORMAL],
			TRUE, 0, 0, w, h);

	/* ramecek */
	gtk_widget_modify_fg (tmp, GTK_STATE_NORMAL, &border_color);
	gdk_draw_rectangle(preview_pixmap,
			tmp->style->fg_gc[GTK_STATE_NORMAL],
			TRUE,
			(w - tw) / 2 - padding - border_width,
			(h - th) / 2 - padding - border_width,
			tw + (padding + border_width) * 2,
			th + (padding + border_width) * 2);
	
	/* pozadi */
	gtk_widget_modify_fg (tmp, GTK_STATE_NORMAL, &bg_color);
	gdk_draw_rectangle(preview_pixmap,
			tmp->style->fg_gc[GTK_STATE_NORMAL],
			TRUE,
			(w - tw) / 2 - padding,
			(h - th) / 2 - padding,
			tw + padding * 2, th + padding * 2);
	
	/* text */
	gtk_widget_modify_fg (tmp, GTK_STATE_NORMAL, &text_color);
	/* gdk_draw_rectangle(preview_pixmap,
			tmp->style->fg_gc[GTK_STATE_NORMAL],
			TRUE,
			40, 40,
			10, 10);
	*/
	gdk_draw_string(preview_pixmap,
			text_font,
			tmp->style->fg_gc[GTK_STATE_NORMAL],
			(w - tw) / 2, (h + th)/2,
			text);

/*
	font =  pango_font_description_from_string("Sans,Medium 8");
	context = gtk_widget_create_pango_context(drawing_area);
	layout = pango_layout_new(context);
	g_object_unref (context);

	pango_layout_set_font_description(layout, font);
	pango_layout_set_text(layout, text, -1);
	gdk_gc_set_foreground(gc, &text_color);
	gdk_draw_layout(drawing_area->window, gc, 30, 30, layout);
	g_object_unref(layout);
*/

	/* Update rect */
	update_rect.x = 0;
	update_rect.y = 0;
	update_rect.width = w;
	update_rect.height = h;
	
	/* vykreslit -- call on_drawingarea_nahled_expose_event() */
	gtk_widget_draw (drawing_area, &update_rect);
}

void update_colors (void) {
	GdkColor color;
	
	gdk_color_parse(css_get_color(main_css, CSS_CTEXT), &color);
	gtk_color_button_set_color(GTK_COLOR_BUTTON(lookup_widget(
					window, "colorbutton_text")), &color);
	
	gdk_color_parse(css_get_color(main_css, CSS_CBG), &color);
	gtk_color_button_set_color(GTK_COLOR_BUTTON(lookup_widget(
					window, "colorbutton_pozadi")), &color);
	
	gdk_color_parse(css_get_color(main_css, CSS_CBORDER), &color);
	gtk_color_button_set_color(GTK_COLOR_BUTTON(lookup_widget(
					window, "colorbutton_ramecek")), &color);

	gdk_color_parse(css_get_color(main_css, CSS_CAROUND), &color);
	gtk_color_button_set_color(GTK_COLOR_BUTTON(lookup_widget(
					window, "colorbutton_okoli")), &color);

}

void update_all (void) {
	update_colors();
	update_kontrast();
	update_css();
	update_preview();
}

void
on_button_help_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *window_help;

	window_help = create_window_help ();
	gtk_widget_show (window_help);
}


void
on_window_destroy                      (GtkObject       *object,
                                        gpointer         user_data)
{
	css_generate_orig(main_css, stdout);
	css_destroy(main_css);
	gtk_main_quit();
}


void
on_colorbutton_text_color_set          (GtkColorButton  *colorbutton,
                                        gpointer         user_data)
{
	GdkColor color;

	gtk_color_button_get_color(colorbutton, &color);
	css_change_color(main_css, CSS_CTEXT, color.red, color.green, color.blue);
	
	update_all();
	
}


void
on_colorbutton_pozadi_color_set        (GtkColorButton  *colorbutton,
                                        gpointer         user_data)
{
	GdkColor color;

	gtk_color_button_get_color(colorbutton, &color);
	css_change_color(main_css, CSS_CBG, color.red, color.green, color.blue);

	update_all();
}


void
on_colorbutton_ramecek_color_set       (GtkColorButton  *colorbutton,
                                        gpointer         user_data)
{
	GdkColor color;

	gtk_color_button_get_color(colorbutton, &color);
	css_change_color(main_css, CSS_CBORDER, color.red, color.green, color.blue);

	update_all();
}


void
on_colorbutton_okoli_color_set         (GtkColorButton  *colorbutton,
                                        gpointer         user_data)
{
	GdkColor color;

	gtk_color_button_get_color(colorbutton, &color);
	css_change_color(main_css, CSS_CAROUND, color.red, color.green, color.blue);

	update_all();
}


void
on_expander_volby_activate             (GtkExpander     *expander,
                                        gpointer         user_data)
{

}


void
on_entry_text_changed                  (GtkEditable     *editable,
                                        gpointer         user_data)
{
	update_preview();
}


void
on_fontbutton_text_font_set            (GtkFontButton   *fontbutton,
                                        gpointer         user_data)
{
	fprintf(stderr, "fnt: %s\n", gtk_font_button_get_font_name(GTK_FONT_BUTTON(
				lookup_widget(window, "fontbutton_text"))));
}


void
on_checkbutton_isramecek_toggled       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	
	draw_border = gtk_toggle_button_get_active(togglebutton);

	css_change_border(main_css, draw_border, border_width);
	
	update_css();
}


void
on_spinbutton_ramecek_changed          (GtkEditable     *editable,
                                        gpointer         user_data)
{
	border_width = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(editable));

	css_change_border(main_css, draw_border, border_width);
	
	update_preview();
	update_css();
}


void
on_expander_kontrast_activate          (GtkExpander     *expander,
                                        gpointer         user_data)
{

}


void
on_expander_css_activate               (GtkExpander     *expander,
                                        gpointer         user_data)
{

}


void
on_button_cancel_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	css_generate_orig(main_css, stdout);
	css_destroy(main_css);
	gtk_main_quit();
}


void
on_button_ok_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	css_generate_file(main_css, stdout);
	css_destroy(main_css);
	gtk_main_quit();
}


gboolean
on_drawingarea_nahled_expose_event     (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
	gdk_draw_pixmap (widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
			preview_pixmap,
			event->area.x, event->area.y,
			event->area.x, event->area.y,
			event->area.width, event->area.height);

	return FALSE;
}


gboolean
on_drawingarea_nahled_configure_event  (GtkWidget       *widget,
                                        GdkEventConfigure *event,
                                        gpointer         user_data)
{
	update_preview();
	return FALSE;
}





void
on_spinbutton_padding_changed          (GtkEditable     *editable,
                                        gpointer         user_data)
{
	padding = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(editable));

	update_preview();
	update_css();
}


gboolean on_stdin_data(GIOChannel *source, GIOCondition condition, gpointer data) {
	gchar *buf;
	gsize len;
	
	
	if (g_io_channel_read_line(source, &buf, &len, NULL, NULL)) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(
				lookup_widget(window, "radiobutton_cssmod")),
				TRUE);
		if (buf) {
			css_parse(main_css, buf);
			update_all();
		}
		g_free(buf);
	}
}



void
on_radiobutton_cssgen_toggled          (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	css_set_update_mode(main_css, !gtk_toggle_button_get_active(togglebutton));
	update_css();
}

