/*
 *
 * css.c  --  css parsing, generating, and misc. functions
 * 
 * Josef Kufner <jkufner@seznam.cz>
 * 
 */

#include <math.h>
#include <ctype.h>
#include "css.h"

/* main structure -- is declared here because
 * it is used in main.c and callbacs.c */
CSS *main_css = NULL;

/* initialize css structure */
CSS *css_init(void) {
	CSS *css;

	css = (CSS *) malloc(sizeof(CSS));
	css->color[CSS_CTEXT].r = 0x00;
	css->color[CSS_CTEXT].g = 0x00;
	css->color[CSS_CTEXT].b = 0x00;

	css->color[CSS_CBG].r = 0xdd;
	css->color[CSS_CBG].g = 0xdd;
	css->color[CSS_CBG].b = 0xdd;
	
	css->color[CSS_CBORDER].r = 0x00;
	css->color[CSS_CBORDER].g = 0x00;
	css->color[CSS_CBORDER].b = 0x00;
	
	css->color[CSS_CAROUND].r = 0xff;
	css->color[CSS_CAROUND].g = 0xff;
	css->color[CSS_CAROUND].b = 0xff;

	css->is_border = 1;
	css->border = 2;
	return css;
}

/* color_buf */
char color_buf[10];

char *css_color_name[] = { "text", "bg", "border", "around" };
const char *prop[] = { "border", "background-color", "color", NULL };

/* parse stream to css structure */
int css_parse(CSS *css, char *buf) {
	int i, r, g, b, x;
	char *fmt[] = { "%1x%1x%1x", "%02x%02x%02x" };
	char *c, *pos, color[7];
	
	strcat(css->data, buf);

	/*** DO NOT TOUCH ***/
	for (i = 0; prop[i]; i++) {
		if ((pos = strstr(buf, prop[i])) && (pos == buf || isspace(*(pos - 1)))
				&& (((x = *(pos + strlen(prop[i]))) == ' ')
					|| x == '\t' || x == ':')) 
		{
			pos += strlen(prop[i]);
			while (*pos == ' ' || *pos == '\t' || *pos == ':' || isalnum(*pos))
				pos++;
			if (*((c = ++pos) - 1) == '#')
				while (pos-c < 6 && isxdigit(*pos))
					color[pos-c] = *(pos++);
			color[pos - c] = '\0';
			if (pos - c == 3 || pos - c == 6) {
				sscanf(color, fmt[(pos - c) / 3 - 1], &r, &g, &b);
				if (pos - c == 3) {
					r += r << 4;
					g += g << 4;
					b += b << 4;
				}
				css->color[2 - i].r = r;
				css->color[2 - i].g = g;
				css->color[2 - i].b = b;
			}
			break;
		}
	}
	
	return 0;
}

/* generate data from css structure */
int css_generate_file(CSS *css, FILE *file) {
	char str[2048];

	if (!css || !file)
		return -1;
	
	css_generate_str(css, str);
	fprintf(file, "%s", str);
	return 0;
}

int css_generate_str(CSS *css, char *str) {
	char *p_prop, *p_val, buf[2048], *p_buf, val[9], *x, y;
	int i;
	
	if (!css || !str)
		return -1;

	if (css->update_mode) {
		/** update original data ***  DO NOT TOUCH ***/
		strcpy(str, css->data);

		for (i = 0; prop[i]; i++) {
			/* buf -> str */
			/* load data */
			strcpy(buf, str);
			strcpy(str, "");
			/* fprintf(stderr, "%d: buf = \"%s\"\n", i, buf); */
			
			/* write back & modify ***  DO NOT TOUCH ***/
			p_buf = buf;
			while ((p_prop = strstr(p_buf, prop[i])) != NULL) {
				if ((p_prop > p_buf && !isspace(*(p_prop - 1))
						&& *(p_prop - 1) != ';')
						|| ((y = *(p_prop + strlen(prop[i]))) != ' '
							&& y != '\t' && y != ':'))
				{
					/* it's part of other property */
					/* fprintf(stderr, "Part !\n"); */
					strncat(str, p_buf, p_prop - p_buf + strlen(prop[i]));
					p_buf = p_prop + strlen(prop[i]);
					continue;
				}

				/*
				fprintf(stderr, "found: \"%s\" at %d\n", prop[i], p_prop - buf);
				fprintf(stderr, "p_prop: \"%s\"\n", p_prop);
				*/

				/* find value */
				if ((p_val = strchr(p_prop, '#'))
						&& ((p_val < (x = strchr(p_prop, ';' ))) || !x)
						&& ((p_val < (x = strchr(p_prop, '\n'))) || !x)
						&& ((p_val < (x = strstr(p_prop, "/*"))) || !x)
						&& ((p_val < (x = strstr(p_prop, "*/"))) || !x)
						&& ((p_val < (x = strstr(p_prop, "//"))) || !x))
				{
					/* fprintf(stderr, "val at +%d\n", p_val-p_prop); */
					
					/* copy all before string */
					/* fprintf(stderr, "2 - p_prop: \"%s\"\n", p_prop); */
					strncat(str, p_buf, p_val - p_buf);
					p_buf = p_val;

					/* append new value */
					sprintf(val, "#%02x%02x%02x",
							css->color[2 - i].r,
							css->color[2 - i].g,
							css->color[2 - i].b);
					strcat(str, val);

					/* skip old value */
					while (isxdigit(*p_buf) || *p_buf == '#')
						p_buf++;

				} else {
					/* value not found */
					/* fprintf(stderr, "val not found\n"); */
					strncat(str, p_buf, p_prop - p_buf);
					strcat(str, prop[i]);
					p_buf = p_prop + strlen(prop[i]);
				}
			}
			
			/* copy end */
			strcat(str, p_buf);
		}
			
	} else {
		/** generate new data **/
		if (css->is_border) {
			sprintf(str,	"color: #%02x%02x%02x;\n"
					"background-color: #%02x%02x%02x;\n"
					"border: %dpx solid #%02x%02x%02x;\n",
					css->color[CSS_CTEXT].r,
					css->color[CSS_CTEXT].g,
					css->color[CSS_CTEXT].b,
					css->color[CSS_CBG].r,
					css->color[CSS_CBG].g,
					css->color[CSS_CBG].b,
					css->border,
					css->color[CSS_CBORDER].r,
					css->color[CSS_CBORDER].g,
					css->color[CSS_CBORDER].b
					);
		} else {
			sprintf(str,	"color: #%02x%02x%02x;\n"
					"background-color: #%02x%02x%02x;\n",
					css->color[CSS_CTEXT].r,
					css->color[CSS_CTEXT].g,
					css->color[CSS_CTEXT].b,
					css->color[CSS_CBG].r,
					css->color[CSS_CBG].g,
					css->color[CSS_CBG].b
					);
		}
	}
	/* fprintf(stderr, "str: \"%s\"\n", str); */
	return 0;
}

/* generate original data */
int css_generate_orig(CSS *css, FILE *file) {

	if (!css)
		return -1;
	
	fprintf(stdout, "%s", css->data);
	return 0;
}

/* change color in css structure */
int css_change_color(CSS *css, int color_id, short r, short g, short b) {
	short red, green, blue;

	if (!css)
		return -1;

	/* recalc values */
	/* TODO -- chyba v zaokrouhlovani */
	red   = (((unsigned short) r) >> 8) & 0xff;
	green = (((unsigned short) g) >> 8) & 0xff;
	blue  = (((unsigned short) b) >> 8) & 0xff;
	
	/* update structure */
	css->color[color_id].r = red;
	css->color[color_id].g = green;
	css->color[color_id].b = blue;
	
	/* debug */
	/*fprintf(stderr, "Color changed: %s = #%02x%02x%02x\n",
			css_color_name[color_id], red, green, blue);*/

	return 0;
}


/* get color in string */
char *css_get_color(CSS *css, int color_id) {
	if (!css)
		return NULL;

	sprintf(color_buf, "#%02x%02x%02x",
			css->color[color_id].r,
			css->color[color_id].g,
			css->color[color_id].b);
	return color_buf;
}


/* change border */
int css_change_border(CSS *css, int is_border, int width) {

	if (!css)
		return -1;

	css->border = (unsigned)width;
	css->is_border = (is_border != 0);
	
	return 0;
}



float css_get_brightness_dif(CSS *css, int color1, int color2) {
	float br1, br2;
	
	br1 = ((float)((css->color[color1].r * 299 + css->color[color1].g * 587
		+ css->color[color1].b * 114)) / 1000.0);
	br2 = ((float)((css->color[color2].r * 299 + css->color[color2].g * 587
		+ css->color[color2].b * 114)) / 1000.0);

	return (br1 - br2);
}

float css_get_color_dif(CSS *css, int color1, int color2) {
	
	return (abs(css->color[color1].r - css->color[color2].r)
		+ abs(css->color[color1].g - css->color[color2].g)
		+ abs(css->color[color1].b - css->color[color2].b));
}


/* update original data ? */
int css_set_update_mode(CSS *css, int mode) {

	if (!css)
		return -1;

	css->update_mode = (mode != 0);
}


/* free memory used by css structure */
int css_destroy(CSS *css) {
	if (css) {
		free(css);
		css = NULL;
		return 0;
	}
	/* else */
	return -1;
}


