/*
 *
 * css.h  --  css parsing, generating, and misc. functions
 * 
 * Josef Kufner <jkufner@seznam.cz>
 * 
 */

#ifndef CSS_H
#define CSS_H

#include <stdio.h>

/* css data structure */
typedef struct {
	/* colors */
	struct {
		short r, g, b;		/* value of color */
		short is_set;		/* is color set ? */
	} color[4];
	int is_border;			/* generate border ? */
	int border;			/* width of border */

	char data[1024];		/* non-parsed original data */
	int update_mode;		/* update orig. data ? */
} CSS;

/* color indexes */
typedef enum {
	CSS_CTEXT = 0, CSS_CBG, CSS_CBORDER, CSS_CAROUND
} CSS_COLOR;

/* color names -- only for debuging */
extern char *css_color_name[];


/* main structure -- is declared here because
 * it is used in main.c and callbacs.c */
extern CSS *main_css;

/* initialize css structure */
CSS *css_init(void);

/* parse stream to css structure */
int css_parse(CSS *css, char *buf);

/* generate data from css structure */
int css_generate_file(CSS *css, FILE *file);
int css_generate_str(CSS *css, char *str);

/* generate original data */
int css_generate_orig(CSS *css, FILE *file);

/* change color in css structure */
int css_change_color(CSS *css, int color_id, short r, short g, short b);

/* get color in string */
char *css_get_color(CSS *css, int color_id);

/* change border */
int css_change_border(CSS *css, int is_border, int width);

/* some extra features ;) */
float css_get_brightness_dif(CSS *css, int color1, int color2);
float css_get_color_dif(CSS *css, int color1, int color2);

/* update original data ? */
int css_set_update_mode(CSS *css, int mode);


/* free memory used by css structure */
int css_destroy(CSS *css);

#endif

